#include <avr/io.h>
#include <avr/eeprom.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

#include "main.h"
#include "globals.h"
#include "functions.h"
#include "cmd_terminal.h"
#include "uart.h"


// UART Terminal
// evaluate commands

void cmd_terminal() {
    uint8_t len;

    len=strlen((char*) gterminal_buffer);

    switch (gterminal_buffer[0]) {
		case 'I':	cmd_get_ID((char*) &gterminal_buffer[2], len); break;
		case 'V':	cmd_get_version((char*) &gterminal_buffer[2], len); break;
		case 'C':	cmd_get_config((char*) &gterminal_buffer[2], len); break;
		case 'S':	cmd_start_timer((char*) &gterminal_buffer[2], len); break;
		case 'P':	cmd_stop_timer((char*) &gterminal_buffer[2], len); break;
		case 'F':	cmd_set_framerate((char*) &gterminal_buffer[2], len); break;
		case 'Y':	cmd_set_sync((char*) &gterminal_buffer[2], len); break;
        case 'O':   cmd_output_f_cal((char*) &gterminal_buffer[2], len); break;
		case 'X':	cmd_cal_fcpu((char*) &gterminal_buffer[2], len); break;
		case 'T':	cmd_set_time((char*) &gterminal_buffer[2], len); break;
        default :   cmd_error(PSTR("unknown command")); break;
	}
    gterminal_buffer[0]=0;
}

// error, print ? + error messeage

void cmd_error(PGM_P error) {
	PRINT("? %S error ?\r\n", error);
}

// command successful, print "OK"

void cmd_OK() {
	uart_puts_P(PSTR("OK\r\n"));
}

// get ID

void cmd_get_ID(char * cmd_buffer, uint8_t len) {
	uart_puts_P(PSTR("FrameDisplay\r\n"));
}

void cmd_get_version(char * cmd_buffer, uint8_t len) {
    PRINT("%01d.%02d\r\n", VERSION_MAJOR, VERSION_MINOR);
}

void cmd_get_config(char * cmd_buffer, uint8_t len) {
    print_cfg();
}

void cmd_start_timer(char* terminal_buffer, uint8_t len) {
    gflag_timer = 1;
    cmd_OK();
}

void cmd_stop_timer(char* terminal_buffer, uint8_t len) {
    gflag_timer = 0;
    cmd_OK();
}

void cmd_set_framerate(char* terminal_buffer, uint8_t len) {
    uint16_t framerate;

    framerate = atoi(terminal_buffer);

    if ( (framerate > 0) && (framerate <=1000) ) {
        gconfig.framerate = framerate;
        set_DDS(framerate);
        cfg_write();
        cmd_OK();
    } else {
        cmd_error(error_parameter);
    }

}

void cmd_cal_fcpu(char* terminal_buffer, uint8_t len) {
    uint32_t f_cpu, delta;

    // user calibration clock is F_CPU/2
    f_cpu = 2*atol(terminal_buffer);
    delta = abs(f_cpu - F_CPU);

    if ( delta < F_CPU/1000 ) {
        gconfig.f_cpu = f_cpu;
        set_DDS(gconfig.framerate);
        cfg_write();
        // timer 2 off
        TCCR2 = 0;
        // SPI init
        SPCR = (1<<SPE) | (1<<MSTR);
        SPSR = (1<<SPI2X);
        cmd_OK();
    } else {
        cmd_error(error_parameter);
    }

}
void cmd_set_sync(char* terminal_buffer, uint8_t len) {

    uint8_t mode = MODE_UNDEFINED;

    switch (terminal_buffer[0]) {
        case 'I' : mode = MODE_INTERNAL; break;
        case 'R' : mode = MODE_EXT_RISE; break;
        case 'F' : mode = MODE_EXT_FALL; break;
    }

    if (mode != MODE_UNDEFINED) {
        gconfig.mode = mode;
        set_SYNC(mode);
        cfg_write();
        cmd_OK();
    } else {
        cmd_error(error_parameter);
    }
}

void cmd_set_time(char* terminal_buffer, uint8_t len) {

    char* next;
    uint8_t hh,mm,ss;
    uint16_t fff;

    next = strtok_P(terminal_buffer, PSTR(":"));
    if (next == NULL) {
        cmd_error(error_parameter);
        return;
    }
    hh=atoi(next);

    next = strtok_P(NULL, PSTR(":"));
    if (next == NULL) {
        cmd_error(error_parameter);
        return;
    }
    mm=atoi(next);

    next = strtok_P(NULL, PSTR(":"));
    if (next == NULL) {
        cmd_error(error_parameter);
        return;
    }
    ss=atoi(next);

    next = strtok_P(NULL, PSTR(":"));
    if (next == NULL) {
        cmd_error(error_parameter);
        return;
    }
    fff=atoi(next);

    //PRINT("DEBUG %02d:%02d:%02d:%03d\r\n", hh, mm, ss, fff);

    if (hh < 24 &&
        mm < 60 &&
        ss < 60 &&
        fff < 1000) {

        gconfig.hh  = hh;
        gconfig.mm  = mm;
        gconfig.ss  = ss;
        gconfig.fff = fff;
        gtime = gconfig;
        time2led();
        TLC59284_update(gTLC59284_data);
        LAT_PULSE
        cfg_write();
        cmd_OK();
    } else {
        cmd_error(error_parameter);
    }
}

void cmd_output_f_cal(char* terminal_buffer, uint8_t len) {

    // SPI OFF
    SPCR = 0;

    gflag_timer = 0;

    // timer 2 init, used for calibration clock output
    // mode 2, CTC, prescaler 1, toggle OCR2 on match 

    TCCR2 = (1<<WGM21) | (1<<COM20) | (1<<CS20);
    OCR2  = 0;       // fastest possible clock, F_CPU / 2

    cmd_OK();
}
