#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include "main.h"
#include "globals.h"
#include "uart.h"

// timer 1, internal timebase

ISR(TIMER1_COMPA_vect) {

    uint16_t static error_accu;
    uint8_t  static prescaler=1;

    prescaler--;
    if (prescaler == 0) {
        LAT_PULSE               // latch new data
        gflag_inc = 1;
        prescaler = gDDSprescaler;        
    }

    // set new timer period (DDS/bresenham frequency counter)

    error_accu += gDDSremainder;
    if (error_accu >= gDDSfrequency) {
        error_accu -= gDDSfrequency;
        OCR1A = gDDSincrement+1;      // long period
    } else {
        OCR1A = gDDSincrement;        // short period
    }

}

// INT1 for external sync pulse counting

ISR(INT1_vect) {

    // latching of new data is done in hardware
    gflag_inc  = 1;
}


// UART receiver interrupt

ISR(USART_RXC_vect) {

    static uint8_t index=0;
    uint8_t tmp;

	tmp=UDR;       						    // get uart data
	if (!gflag_terminal) {
        if (tmp=='\r') {					// CR terminates command
    		gterminal_buffer[index]=0;		// string terminator
    	    index=0;
            gflag_terminal=1;
    	} else {
    		gterminal_buffer[index]=tmp;
    		if (index < (sizeof(gterminal_buffer)-1)) index++;	// prevent overrun
    	}
    }  
}

