/*

Precision frame counter

ATmega8 @ 8MHz

*/

// includes

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <avr/eeprom.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include "main.h"
#include <util/delay.h>
#include "globals.h"
#include "uart.h"
#include "functions.h"
#include "cmd_terminal.h"

static FILE mystdout = FDEV_SETUP_STREAM(putchar__, 0, _FDEV_SETUP_WRITE);

// main

int main(void) {

    int i, j;

    // variables init

	// redirect stdout for use of PRINT macro
	stdout = &mystdout;

    SYNC_INTERNAL
    BLANK_HIGH

    // IO init
    DDRD  = ~ ((1<<PD3) | (1<<PD0));
    DDRB  = ~(1<<PB4);

    // SPI init

    SPCR = (1<<SPE) | (1<<MSTR);
    SPSR = (1<<SPI2X);

    // UART config

    UCSRB = (1<<RXCIE) | (1<<RXEN) | (1<<TXEN); 
    UBRRH = UBRR_VAL >> 8;
    UBRRL = UBRR_VAL;
    
    // timer 1 init, frame rate generator
    // mode 4, CTC, prescaler 1
    // period is defined by OCR1A

    TCCR1B = (1<<WGM12) | (1<<CS10);

    // init external INT1 interrupt, rising edge

    MCUCR = (1<<ISC11) | (1<<ISC10);
 
    PRINT("\r\nPower Up\r\n");

    cfg_read();
    set_DDS(gconfig.framerate);
    set_SYNC(gconfig.mode);
    print_cfg();
    if (gconfig.mode != MODE_INTERNAL) gflag_timer = 1;

    BLANK_LOW

    // power on self test
    // display numbers 0-9 on all digits

    for(i=0; i<10; i++) {
        for(j=0; j<9; j++) {
            gTLC59284_data[j]=hex2led[i];
        }
        TLC59284_update(gTLC59284_data);
        LAT_PULSE
        _delay_ms(200);
    }

    // some fun ;-)

    animation((uint16_t*)animation1, ANIMATION_SIZE);

    // load start time

    time2led();
    TLC59284_update(gTLC59284_data);
    LAT_PULSE

    // interrupt enable

    sei();

    // endless main loop

    while (1) {
        if (gflag_terminal) {
            cmd_terminal();
            gflag_terminal=0;
        }   
        
        if (gflag_inc) {
            gflag_inc=0;
            if (gflag_timer) {
                increment_timer();
                time2led();
                TLC59284_update(gTLC59284_data);
            }
        }         
    }
}
